@@ -37,13 +37,12 @@ import Hedgehog.Internal.Runner as H
 import Hedgehog.Internal.Report
 import Hedgehog.Internal.Seed as Seed
 
-data HP = HP PropertyName Property
+data HP = HP T.TestName (Maybe PropertyName) Property
   deriving (Typeable)
 
 -- | Create a 'T.TestTree' from a Hedgehog 'Property'.
-{-# DEPRECATED testProperty "testProperty will cause Hedgehog to provide incorrect instructions for re-checking properties" #-}
 testProperty :: T.TestName -> Property -> T.TestTree
-testProperty name prop = T.singleTest name (HP (PropertyName name) prop)
+testProperty name prop = T.singleTest name (HP name Nothing prop)
 
 -- | `testPropertyNamed` @testName propertyName property@ creates a
 -- 'T.TestTree' from @property@ using @testName@ as the displayed
@@ -61,7 +60,8 @@ testProperty name prop = T.singleTest name (HP (PropertyName name) prop)
 --
 -- @since 1.2.0.0
 testPropertyNamed :: T.TestName -> PropertyName -> Property -> T.TestTree
-testPropertyNamed name propName prop = T.singleTest name (HP propName prop)
+testPropertyNamed name propName prop =
+  T.singleTest name (HP name (Just propName) prop)
 
 -- | Create a 'T.TestTree' from a Hedgehog 'Group'.
 fromGroup :: Group -> T.TestTree
@@ -73,15 +73,15 @@ fromGroup group =
     mkTestTree (propName, prop) = testProperty (unPropertyName propName) prop
 
 -- | The replay token to use for replaying a previous test run
-newtype HedgehogReplay = HedgehogReplay (Maybe (Size, Seed))
+newtype HedgehogReplay = HedgehogReplay (Maybe (Skip, Seed))
   deriving (Typeable)
 
 instance IsOption HedgehogReplay where
   defaultValue = HedgehogReplay Nothing
   parseValue v = HedgehogReplay . Just <$> replay
-    -- Reads a replay token in the form "{size} {seed}"
-    where replay = (,) <$> safeRead (unwords size) <*> safeRead (unwords seed)
-          (size, seed) = splitAt 2 $ words v
+    -- Reads a replay token in the form "{skip} {seed}"
+    where replay = (,) <$> skipDecompress (unwords skip) <*> safeRead (unwords seed)
+          (skip, seed) = splitAt 1 $ words v
   optionName = return "hedgehog-replay"
   optionHelp = return "Replay token to use for replaying a previous test run"
 
@@ -147,7 +147,10 @@ propertyTestLimit =
 reportToProgress :: PropertyConfig
                  -> Report Progress
                  -> T.Progress
-reportToProgress config (Report testsDone _ _ status) =
+reportToProgress config Report{
+    reportTests = testsDone,
+    reportStatus = status
+  } =
   let
     TestLimit testLimit = propertyTestLimit config
     ShrinkLimit shrinkLimit = propertyShrinkLimit config
@@ -162,22 +165,27 @@ reportToProgress config (Report testsDone _ _ status) =
 
 reportOutput :: Bool
              -> UseColor
-             -> PropertyName
+             -> T.TestName
+             -> Maybe PropertyName
              -> Report Result
              -> IO String
-reportOutput showReplay useColor name report = do
-  s <- renderResult useColor (Just name) report
+reportOutput showReplay useColor testName name report = do
+  s <- renderResult useColor name report
   pure $ case reportStatus report of
     Failed fr ->
       let
-        size = failureSize fr
-        seed = failureSeed fr
+        count = reportTests report
+        seed = reportSeed report
         replayStr =
           if showReplay
           then
-            "\nUse '--hedgehog-replay \"" ++
-            show size ++ " " ++ show seed ++
-            "\"' to reproduce."
+            "\nUse \"--pattern \'$NF ~ /" ++
+            testName ++
+            "/\' --hedgehog-replay \'" ++
+            skipCompress (SkipToShrink count $ failureShrinkPath fr) ++
+            " " ++
+            show seed ++
+            "\'\" to reproduce from the command-line."
           else ""
       in
         s ++ replayStr ++ "\n"
@@ -193,7 +201,7 @@ instance T.IsTest HP where
            , Option (Proxy :: Proxy HedgehogShrinkRetries)
            ]
 
-  run opts (HP name (Property pConfig pTest)) yieldProgress = do
+  run opts (HP testName name (Property pConfig pTest)) yieldProgress = do
     useColor <- detectColor
     let
       HedgehogReplay         replay = lookupOption opts
@@ -208,18 +216,17 @@ instance T.IsTest HP where
           (fromMaybe (propertyShrinkLimit pConfig) mShrinks)
           (fromMaybe (propertyShrinkRetries pConfig) mRetries)
           (NoConfidenceTermination $ fromMaybe (propertyTestLimit pConfig) mTests)
+          (maybe Nothing (Just . fst) replay)
 
     randSeed <- Seed.random
-    let
-      size = maybe 0 fst replay
-      seed = maybe randSeed snd replay
+    let seed = maybe randSeed snd replay
 
-    report <- checkReport config size seed pTest (yieldProgress . reportToProgress config)
+    report <- checkReport config 0 seed pTest (yieldProgress . reportToProgress config)
 
     let
       resultFn = if reportStatus report == OK
                  then T.testPassed
                  else T.testFailed
 
-    out <- reportOutput showReplay useColor name report
+    out <- reportOutput showReplay useColor testName name report
     return $ resultFn out
